import types
import logging

from PyQt5.Qt import QObject

import pqaut.automator.factory as factory


logger = logging.getLogger(__name__)

class QObjectAutomator(object):

    def __init__(self, target):
        self._target = target

    @property
    def target(self):
        return self._target

    def click(self):
        pass

    def clickable_target(self, point):
        return _target

    def is_offscreen(self):
        return False

    def get_children(self):
        children = []

        try:
            children = self._target.findChildren(QObject)
        except Exception as ex:
            logger.debug('could not find children on {}: {}'.format(self._target, ex))

        logger.debug('found children {} on {}'.format(self._target, children))
        return [factory.automate(c) for c in children];

    def is_match(self, value, automation_type=None):
        return False

    def automation_id(self):
        return self.value_or_default('automation_id', None)

    def automation_type(self):
        return self.value_or_default('automation_type', '')

    def get_name(self):
        return self.value_or_default('objectName', '')

    def get_value(self):
        return self.value_or_default('text', '')

    def to_json(self, is_recursive=False):
        json={ 'type':self._target.__class__.__name__, 
               'value':self.value_or_default('text', ''),  }

        if is_recursive:
            children_json = []
            for child in self.get_children():
                children_json.append(child.to_json(is_recursive))
            json['children'] = children_json

        return json

    def hasmethod(self, method_name):
        return hasattr(self._target, method_name) and callable(getattr(self._target, method_name))

    def value_or_default(self, value_name, default):
        value = default
        if self.hasmethod(value_name):
            method = getattr(self._target, value_name)
            try:
                value = method()
            except Exception as e:
                pass

        elif hasattr(self._target, value_name):
            value = getattr(self._target, value_name)

        if type(value) == types.StringType:
            return value.decode('utf-8')

        return value



import requests
import time
import json
from lettuce import world
from nose.tools import assert_equals, assert_true, assert_is_not_none, assert_is_none, assert_false


def wait_for_automation_server():
    while (True):
        time.sleep(0.5)
        try:
            response = requests.get("http://localhost:5123/ping")
            if response.status_code == 200:
                break
        except:
            pass

def tap(name, automation_type = None):
    assert_is_visible(name, automation_type)
    time.sleep(0.4)
    headers = {'Content-type': 'application/json', 'Accept': 'application/json' }
    body = {'query':{'window_name':'', 'value':name, 'automation_type':automation_type}}
    response = requests.post('http://localhost:5123/click', data=json.dumps(body), headers=headers)
    time.sleep(0.3)

def assert_is_not_visible(name, automation_type = None, timeout=3):
    assert_true(_wait_for_element_to_not_be_visible(name, automation_type, timeout), u"Found element but expected it to not be visible")

def assert_is_visible(name, automation_type = None, timeout=3):
    assert_is_not_none(_wait_for_visible_element(name, automation_type, timeout), u"Visible element not found with name {0} and type {1}".format(name, automation_type))

def _wait_for_element_to_not_be_visible(name, automation_type, timeout):
    retries = 0
    delay = 0.3
    while retries * delay < timeout:
        retries += 1

        try:
            found_element = _find_element(name, automation_type)
        except requests.exceptions.ConnectionError:
            time.sleep(delay)
            continue

        if found_element is None or not _element_is_visible(found_element):
            return True

        time.sleep(delay)

    return False

def _wait_for_visible_element(name, automation_type, timeout):
    retries = 0
    delay = 0.3
    while retries * delay < timeout:
        retries += 1

        try:
            found_element = _find_element(name, automation_type)
        except requests.exceptions.ConnectionError:
            time.sleep(delay)
            continue

        if found_element is not None and _element_is_visible(found_element):
            return found_element

        time.sleep(delay)

    return None

def _element_is_visible(element):
    return element["visible"] and not element["offscreen"]

def _find_element(name, automation_type):
    headers = {'Content-type': 'application/json', 'Accept': 'application/json' }
    body = {'query':{'window_name':'', 'value':name, 'automation_type':automation_type}}
    response = requests.post('http://localhost:5123/find_element', data=json.dumps(body), headers=headers)
    found_element = response.json()
    if len(found_element.keys()) == 0:
        return None

    return found_element




import pqaut.automator.factory as factory
import pqaut.automator.qwidget


class QQuickWidgetAutomator(pqaut.automator.qwidget.QWidgetAutomator):

    def __init__(self, target):
        self._target = target

    def get_children(self):
        children = []
        root_object = self._target.rootObject()
        if root_object is not None:
            children.append(factory.automate(root_object))
        return children



import sys
try:
    import ez_setup
    ez_setup.use_setuptools()
except ImportError:
    pass

from setuptools import setup

setup(
    name='pqaut',
    version='0.1.9',
    author='Gary Johnson',
    author_email = 'gary@gjtt.com',
    description = 'Automation for BDD testing PyQt apps',
    install_requires=['bottle'],
    tests_require=['bottle'],
    license = 'MIT License',
    packages = ['pqaut', 'pqaut.automator'],
    )



import types

import pqaut.automator.factory as factory
import pqaut.automator.qobject


class QQuickWindowAutomator(pqaut.automator.qobject.QObjectAutomator):

    def __init__(self, target):
        self._target = target

    def get_children(self):
        return [factory.automate(self._target.contentItem())]
